/****************************************************************************\
**  Title:       DCX2PCX.C                                                  **
**  Purpose:     Extract PCX image files from a DCX image file.             **
**  Version:     1.0                                                        **
**  Date:        April 1991                                                 **
**  Author:      James D. Murray, Anaheim, CA  USA                          **
**  C Compilers: Borland C++ v2.0, Microsoft C v6.00a                       **
**                                                                          **
**  DCX2PCX extracts all the PCX images from within a DCX file.  A DCX file **
**  may contain up to 1023 PCX images.  Each PCX image is a complete PCX    **
**  image file, including header, palette, and compressed image data.       **
**                                                                          **
**  Copyright (C) 1991 by Graphics Software Labs.  All rights reserved.     **
\****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "endianio.h"
#include "pcx.h"

DWORD (*GetDword)(FILE *);      /* Pointer to ENDIANIO function */

#define BUFFERSIZE  8192        /* Size of copy buffer */


int
main(argc, argv)
int argc;
char *argv[];
{
    register WORD i, j;         /* Loop counters                            */
    FILE *fpDcx;                /* DCX file input file stream               */
    FILE *fpPcx;                /* PCX file output file stream              */
    int  counter = 0;           /* Counter used to construct PCX file names */
    int  retsize;               /* fread() return size value holder         */
    int  numberOfBlockReads;    /* Number of block to fread() in DCX file   */
    int  remainder;             /* Remainder of data to fread()             */
    long lpcxFileSize;          /* Size of a PCX file                       */
    char pcxFileName[20];       /* PCX file name holder                     */
    char dcxFileName[20];       /* DCX file name holder                     */
    char buffer[BUFFERSIZE];    /* Copy buffer                              */
    DCXHEADER dcxHead;          /* DCX header structure                     */

    puts("DCX2PCX - Extract PCX image file(s) from a DCX file (v1.00)\n\n");

    /* Check for proper number of command line arguments */
    if (argc < 2)
    {
        fputs("Usage: DCX2PCX filename.DCX", stderr);
        exit(-1);
    }

    strcpy(dcxFileName, argv[1]);   
    if (!strrchr(dcxFileName, '.'))     /* Check if the file name has an    */
        strcat(dcxFileName, ".DCX");    /* extension and append .DCX if not */

    /* Open the DCX image file */
    if ((fpDcx = fopen(dcxFileName, "rb")) == NULL)
    {
        fprintf(stderr, "DCX2PCX: Cannot open file %s\n", dcxFileName);
        exit(-2);
    }

    /* Read using little-endian byte order */
    GetDword = GetLittleDword;

    /* Read the ID word */
    dcxHead.Id = GetDword(fpDcx);

    /* Check the magic number */
    if (dcxHead.Id != 987654321L)
    {
        fprintf(stderr, "DCX2PCX: %s is not a DCX-format file!\n",
          dcxFileName);
        exit(-3);
    }

    /* Zero-out page list */
    memset(&dcxHead.PageList, 0, sizeof(dcxHead.PageList));

    /* Read the page list from the DCX file */
    i = 0;
    do
    {
        dcxHead.PageList[i] = GetDword(fpDcx);
    }
    while (dcxHead.PageList[i++] != (DWORD) 0);

    /* For each PCX image in the DCX file ... */
    for (i = 0; dcxHead.PageList[i]; i++)
    {
        /*
        ** The DCX format does not allow a way to preserve the orignal
        ** file names of the PCX images, so we must make new file names.
        **
        ** Names may range from PCX0000.PCX to PCX1022.PCX for a total
        ** of 1023 PCX image files.
        */
        sprintf(pcxFileName, "PCX%04d.PCX", counter);
        if ((fpPcx = fopen(pcxFileName, "wb")) == NULL)
        {
            fprintf(stderr, "DCX2PCX: Cannot create PCX file %s\n",
              pcxFileName);
            break;
        }
        printf("Extracting PCX image file %-13s", pcxFileName);

        /* Seek to the next PCX image */
        fseek(fpDcx, dcxHead.PageList[i], SEEK_SET);

        /* If this next PCX image is not the last PCX image in the file ... */
        if (dcxHead.PageList[i + 1] != 0)
        {
            /*
            ** Calculate the number of blocks in a PCX image based on
            ** the value of the BUFFERSIZE identifier.
            */
            lpcxFileSize = dcxHead.PageList[i+1] - dcxHead.PageList[i];
            numberOfBlockReads = (int) (lpcxFileSize / (long) BUFFERSIZE);
            remainder = (int) (lpcxFileSize % (long) BUFFERSIZE);

            printf("  Size = %ld\n", lpcxFileSize);

            /* Read and write blocks of data */
            for (j = 0; j < numberOfBlockReads; j++)
            {
                if ((retsize = fread(&buffer, sizeof(char), BUFFERSIZE, fpDcx))
                  == sizeof(buffer))
                    fwrite(&buffer, sizeof(char), BUFFERSIZE, fpPcx);
            }
            fread(&buffer, sizeof(char), remainder, fpDcx);
            fwrite(&buffer, sizeof(char), remainder, fpPcx);

            /* Check if file pointer is where its suppose to be */
            if (ftell(fpDcx) != dcxHead.PageList[i + 1])
                fprintf(stderr, "DCX2PCX: File position error!\
                  (are: %ld should be: %ld)\n",
                  ftell(fpDcx), dcxHead.PageList[i + 1]);

        }
        else    /* Extract last PCX image in the DCX file */
        {   
            while ((retsize = fread(buffer, sizeof(char), BUFFERSIZE, fpDcx))
              == sizeof(buffer))
                fwrite(&buffer, sizeof(char), BUFFERSIZE, fpPcx);
            fwrite(&buffer, sizeof(char), retsize, fpPcx);
            printf("  Size = %ld\n", ftell(fpDcx) - dcxHead.PageList[i]);
        }
        fclose(fpPcx);      /* Close the PCX file             */
        counter++;          /* Increment the PCX file counter */
    }
    fclose(fpDcx);          /* Close the DCX file             */

    printf("\n%d PCX image files extracted.\n\n", counter);

    return(0);              /* Successful termination         */
}
}
